/**
 * @file
 * @details Support FPv4 and FPv5 for ARMv7-M
 *
 * @date 13.06.2018
 * @author Alexander Kalmuk
 */

#include <stdint.h>

#include <util/log.h>
#include <util/math.h>
#include <arm/fpu.h>
#include <hal/reg.h>

#include <embox/unit.h>
#include <framework/mod/options.h>

EMBOX_UNIT_INIT(fpv5_init);

static int fpv5_init(void) {
	uint32_t mvfr0, fpccr;

	/* Enable FPU */
	/** FIXME Currently FPU is enabled permanently */
	REG_STORE(CPACR, REG_LOAD(CPACR) | (0xf << 20));

	/* Disable FPU context preservation/restoration on exception
	 * entry and exit, because we can guarantee every irq handler
	 * execute without using FPU */
	REG_STORE(FPCCR, REG_LOAD(FPCCR) & ~(0x3 << 30));

	mvfr0 = REG_LOAD(MVFR0);
	fpccr = REG_LOAD(FPCCR);

	log_boot_start();
	log_boot("FPv4/FPv5 info:\n"
			 "\t\t\t MVFR0 (Media and VFP Feature Register 0) = 0x%08x\n"
			 "\t\t\t\t Single precision support = %s\n"
			 "\t\t\t\t Double precision support = %s\n"
			 "\t\t\t\t Size of FP register bank = %s\n"
			 "\t\t\t MVFR1 (Media and VFP Feature Register 1) = 0x%08x\n"
			 "\n"
			 "\t\t\tFPU registers:\n"
			 "\t\t\t CPACR (Coprocessor Access Control Register)     = 0x%08x\n"
			 "\t\t\t FPCCR (Floating-point Context Control Register) = 0x%08x\n"
			 "\t\t\t\t Automatic hardware state preservation and restoration\n"
			 "\t\t\t\t on exception entry and exit       = %s\n"
			 "\t\t\t\t Automatic lazy state preservation = %s\n",
			 mvfr0,
			 (((mvfr0 >> 4) & 0xf) == 0x2) ? "yes" : "no",
			 (((mvfr0 >> 8) & 0xf) == 0x0) ? "no" : "Unknown",
			 (((mvfr0 >> 0) & 0xf) == 0x1) ? "16 x 64-bit registers" : "Unknown",
			 REG_LOAD(MVFR1),
			 REG_LOAD(CPACR),
			 fpccr,
			 (fpccr & (1 << 31)) ? "yes" : "no",
			 (fpccr & (1 << 30)) ? "yes" : "no"
	);
	log_boot_stop();

	return 0;
}
