/**
 * @file io.h
 * @brief
 * @author Denis Deryugin <deryugin.denis@gmail.com>
 * @version
 * @date 18.12.2017
 */

#ifndef E2K_IO_H_
#define E2K_IO_H_

#include <stdint.h>

#include <e2k_mas.h>
#include <e2k_api.h>
#include <asm/hal/memory_barrier.h>

#if 0
#define WMB_AFTER_ATOMIC	".word 0x00008001\n" \
				".word 0x30000084\n"
#endif

#define __e2k_atomic32_add(__val, __addr) \
({ \
	int __rval; \
	asm volatile ("\n1:" \
			  "\n\tldw,0 %[addr], %[rval], mas=0x7" \
			  "\n\tadds %[rval], %[val], %[rval]" \
			  "\n\t{"\
			  "\n\tstw,2 %[addr], %[rval], mas=0x2" \
			  "\n\tibranch 1b ? %%MLOCK" \
			  "\n\t}" \
			  WMB_AFTER_ATOMIC \
			  : [rval] "=&r" (__rval), [addr] "+m" (*(__addr)) \
			  : [val] "ir" (__val) \
			  : "memory"); \
	__rval; \
})

static inline void e2k_wait_all(void) {
	_Pragma ("no_asm_inline")
	asm volatile ("wait \ttrap = %0, ma_c = %1, fl_c = %2, ld_c = %3, "
			"st_c = %4, all_e = %5, all_c = %6"
			: : "i" (0), "i" (1), "i" (1), "i" (0), "i" (0), "i" (1), "i" (1) : "memory");
}
#if 0
#define _sal	0x100	/* store-after-load modifier for _lt_c */
#define _sas	0x80	/* store-after-store modifier for _st_c */
#define _trap	0x40	/* stop the conveyor untill interrupt */
#define _ma_c	0x20
#define _fl_c	0x10
#define _ld_c	0x8	/* stop until all load operations complete */
#define _st_c	0x4	/* stop until store operations complete */
#define _all_e	0x2
#define _all_c	0x1

#define E2K_WAIT(num) \
({ \
	asm volatile ("{wait ma_c=%5, fl_c=%4, "\
			"ld_c = %3, st_c=%2, all_e=%1, all_c=%0}" \
		      : \
		      : "i" (((num) & 0x1)), \
			"i" (((num) & 0x2)  >> 1), \
			"i" (((num) & 0x4)  >> 2), \
			"i" (((num) & 0x8)  >> 3), \
			"i" (((num) & 0x10) >> 4), \
			"i" (((num) & 0x20) >> 5) \
		      : "memory" ); \
})
#endif

#if 0
#define mb()	E2K_WAIT(_st_c | _ld_c)
#define rmb()	E2K_WAIT(_ld_c)

static inline void wmb() {
	_Pragma ("no_asm_inline")
	asm volatile (	".word 0x00008001\n"
			".word 0x30000084\n"
			::: "memory");
}
#endif

static inline void e2k_write8(uint8_t val, void *addr) {
	E2K_WRITE_MAS_B((unsigned long)addr, val, MAS_IOADDR);
#if 0
       asm volatile ("stb,2\t0x0, [%0] %2, %1" :
		: "r" (addr), "r" (val), "i" (MAS_IOADDR) : "memory");
       wmb();
#endif
}

static inline void e2k_write16(uint16_t val, void *addr) {
	E2K_WRITE_MAS_H((unsigned long)addr, val, MAS_IOADDR);
#if 0
       asm volatile ("sth,2\t0x0, [%0] %2, %1" :
		: "r" (addr), "r" (val), "i" (MAS_IOADDR) : "memory");
#endif

}

static inline void e2k_write32(uint32_t val, void *addr) {
	E2K_WRITE_MAS_W((unsigned long)addr, val, MAS_IOADDR);
#if 0
       asm volatile ("stw,2\t0x0, [%0] %2, %1" :
		: "r" (addr), "r" (val), "i" (MAS_IOADDR) : "memory");
#endif
}

static inline void e2k_write64(uint64_t val, void *addr) {
	E2K_WRITE_MAS_D((unsigned long)addr, val, MAS_IOADDR);
#if 0
       asm volatile ("std,2\t0x0, [%0] %2, %1" :
		: "r" (addr), "r" (val), "i" (MAS_IOADDR) : "memory");
#endif
}

static inline uint8_t e2k_read8(void *addr) {
	return E2K_READ_MAS_B((unsigned long)addr, MAS_IOADDR);
#if 0
	register uint8_t res;
	asm volatile ("ldb,2 \t0x0, [%1] %2, %0" :
		"=r" (res) : "r" (addr), "i" (MAS_IOADDR));
	wmb();
	return res;
#endif
}

static inline uint16_t e2k_read16(void *addr) {
	return E2K_READ_MAS_H((unsigned long)addr, MAS_IOADDR);
#if 0
	register uint16_t res;
	asm volatile ("ldh,2 \t0x0, [%1] %2, %0" :
		"=r" (res) : "r" (addr), "i" (MAS_IOADDR));
	return res;
#endif
}

static inline uint32_t e2k_read32(void *addr) {
	return E2K_READ_MAS_W((unsigned long)addr, MAS_IOADDR);
#if 0
	register uint32_t res;
	asm volatile ("ldw,2 \t0x0, [%1] %2, %0" :
		"=r" (res) : "r" (addr), "i" (MAS_IOADDR));
	return res;
#endif
}

static inline uint64_t e2k_read64(void *addr) {
	return E2K_READ_MAS_D((unsigned long)addr, MAS_IOADDR);
#if 0
	register uint64_t res;
	asm volatile ("ldd,2 \t0x0, [%1] %2, %0" :
		"=r" (res) : "r" (addr), "i" (MAS_IOADDR));
	return res;
#endif
}

/* x86 port simulation */
#define E2K_X86_IO_PORT_BASE	0xfff0000000UL

static inline void e2k_out8(uint8_t val, int port) {
	e2k_write8(val, (void *)(E2K_X86_IO_PORT_BASE + port));
}

static inline uint8_t e2k_in8(int port) {
	return e2k_read8((void *)(E2K_X86_IO_PORT_BASE + port));
}

#endif /* E2K_IO_H_ */
